//  pcanbasic_jni.cpp
//
//  ~~~~~~~~~~~~
//
//  PCANBasic JNI Interface
//
//  ~~~~~~~~~~~~
//
//  ------------------------------------------------------------------
//  Author : Jonathan Urban
//  Co-Author : Thomas Haber (thomas@toem.de) for Linux port
//  Last change: 18.06.2010
//
//  Language: C++
//  ------------------------------------------------------------------
//
//  Copyright (C) 1999-2010  PEAK-System Technik GmbH, Darmstadt
//  more Info at http://www.peak-system.com
//  ------------------------------------------------------------------
//
// linux@peak-system.com
// www.peak-system.com
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//

#ifndef _PCANBasic_JNI_
#define _PCANBasic_JNI_

#include <stdio.h>
#include "jni.h"


// Throws a given exception in JVM
void ThrowExByName(JNIEnv *env, const char *name, const char *msg);
// Get Java Class Enum Value
int GetClassEnumValue(JNIEnv *env, jobject* target, const char *className, const char *valueName, const char *byteCodeTypeName);
// Initializes the pointers for the PCANBasic functions
void InitializePointers();
// Instanciates/loads all functions within a loaded DLL
void LoadAPI();
// Releases a loaded API
void UnloadAPI();
// Parse ANSI-C TPCANStatus to the corresponding Java TPCANHandle
void ParseTPCANHandleToJava(JNIEnv *env, TPCANHandle source, jobject* target);
// Parse a Java Enum value to int
int ParseEnumValueFromJava(JNIEnv *env, jobject* source, const char *type);
// Appends a string at the end of a Java StringBuffer
void appendTextToJavaStringBuffer(JNIEnv *env, jobject stringBuffer, char* value);
// Copy TPCANMsg object to a Java CANMessage object
int ParseCanMessageToJava(JNIEnv *env, TPCANMsg* msg, jobject* jCANMessage);
void ParseTPCANStatusToJava(JNIEnv *env, TPCANStatus source, jobject* target);

// Parameter which stores Thread info
typedef struct {
  int   channel;
  pthread_t thread;
  int terminating;
} CANReadThreadFuncParameter;
// Handle to the thread to read using Received-Event method
static CANReadThreadFuncParameter* threadParameterList[0x100];
// Pointer to the loaded Java Virtual Machine
JavaVM *m_vm;


#endif
